"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CertificateErrors = void 0;
var CertificateErrors;
(function (CertificateErrors) {
    let Issuance;
    (function (Issuance) {
        class CertificateAlreadyIssued extends Error {
            constructor(internalCertificateId) {
                super(`[CRITICAL ERROR] Issuance for internalCertificateId: ${internalCertificateId} failed. Certificate already issued.`);
            }
        }
        Issuance.CertificateAlreadyIssued = CertificateAlreadyIssued;
    })(Issuance = CertificateErrors.Issuance || (CertificateErrors.Issuance = {}));
    let Transfer;
    (function (Transfer) {
        class FromZeroAddress extends Error {
            constructor(internalCertificateId) {
                super(`Transfer for internalCertificateId: ${internalCertificateId} failed. Transfer is from zero address(0x0).`);
            }
        }
        Transfer.FromZeroAddress = FromZeroAddress;
        class ToZeroAddress extends Error {
            constructor(internalCertificateId) {
                super(`Transfer for internalCertificateId: ${internalCertificateId} failed. Transfer is to zero address(0x0).`);
            }
        }
        Transfer.ToZeroAddress = ToZeroAddress;
        class NotEnoughBalance extends Error {
            constructor(internalCertificateId, address) {
                super(`Transfer for: ${internalCertificateId} failed. Address: ${address} has not enough balance.`);
            }
        }
        Transfer.NotEnoughBalance = NotEnoughBalance;
    })(Transfer = CertificateErrors.Transfer || (CertificateErrors.Transfer = {}));
    let Claim;
    (function (Claim) {
        class ForZeroAddress extends Error {
            constructor(internalCertificateId) {
                super(`Claim for internalCertificateId: ${internalCertificateId} failed. Transfer is for zero address(0x0).`);
            }
        }
        Claim.ForZeroAddress = ForZeroAddress;
        class NotEnoughBalance extends Error {
            constructor(internalCertificateId, address) {
                super(`Claim for: ${internalCertificateId} failed. Address: ${address} has not enough balance.`);
            }
        }
        Claim.NotEnoughBalance = NotEnoughBalance;
    })(Claim = CertificateErrors.Claim || (CertificateErrors.Claim = {}));
    class BatchError extends Error {
        constructor(originError) {
            super(`Batch operation failed. Failing operation message: ${originError.message}`);
        }
    }
    CertificateErrors.BatchError = BatchError;
    class UnknownEventType extends Error {
        constructor(internalCertificateId, eventType) {
            super(`[CRITICAL ERROR] Unknown event type ${eventType} for certificate ${internalCertificateId}.`);
        }
    }
    CertificateErrors.UnknownEventType = UnknownEventType;
    class FirstCertificateEventIsNotIssuance extends Error {
        constructor(internalCertificateId, type) {
            super(`[CRITICAL ERROR] First event for certificate: ${internalCertificateId} is not issuance but ${type}.`);
        }
    }
    CertificateErrors.FirstCertificateEventIsNotIssuance = FirstCertificateEventIsNotIssuance;
    class CertificateNoEvents extends Error {
        constructor() {
            super(`[CRITICAL ERROR] Tried to create certificate without events.`);
        }
    }
    CertificateErrors.CertificateNoEvents = CertificateNoEvents;
    class CertificateTooManyPersisted extends Error {
        constructor(internalCertificateId, toPersistCounter) {
            super(`[CRITICAL ERROR] Certificate ${internalCertificateId} has too many (${toPersistCounter}) persisted events.`);
        }
    }
    CertificateErrors.CertificateTooManyPersisted = CertificateTooManyPersisted;
})(CertificateErrors = exports.CertificateErrors || (exports.CertificateErrors = {}));
//# sourceMappingURL=errors.js.map