import { IClaimCommand, IIssueCommand, ITransferCommand } from '../../types';
export declare const isTransferEvent: (e: ICertificateEvent) => e is CertificateTransferredEvent;
export declare const isIssueEvent: (e: ICertificateEvent) => e is CertificateIssuedEvent<unknown>;
export declare const isIssuePersistedEvent: (e: ICertificateEvent) => e is CertificateIssuancePersistedEvent;
export declare const isClaimEvent: (e: ICertificateEvent) => e is CertificateClaimedEvent;
export declare const isPersistedEvent: (e: ICertificateEvent) => e is PersistedEvent;
export declare enum CertificateEventType {
    Issued = "Issued",
    Transferred = "Transferred",
    Claimed = "Claimed",
    IssuancePersisted = "IssuancePersisted",
    TransferPersisted = "TransferPersisted",
    ClaimPersisted = "ClaimPersisted",
    PersistError = "PersistError"
}
export interface ICertificateEvent {
    id: number;
    type: CertificateEventType;
    version: number;
    internalCertificateId: number;
    payload: unknown;
    createdAt: Date;
}
export declare type PersistedEvent = CertificateIssuancePersistedEvent | CertificateClaimPersistedEvent | CertificateTransferPersistedEvent | CertificatePersistErrorEvent;
export declare class CertificateIssuedEvent<MetadataType = unknown> implements ICertificateEvent {
    private readonly _id;
    readonly internalCertificateId: number;
    readonly payload: IIssueCommand<MetadataType>;
    readonly type = CertificateEventType.Issued;
    readonly version = 1;
    readonly createdAt: Date;
    private constructor();
    get id(): number;
    static createNew<T = unknown>(internalCertificateId: number, payload: IIssueCommand<T>): CertificateIssuedEvent<T>;
}
export declare class CertificateTransferredEvent implements ICertificateEvent {
    private readonly _id;
    readonly internalCertificateId: number;
    readonly payload: ITransferCommand;
    readonly type = CertificateEventType.Transferred;
    readonly version = 1;
    readonly createdAt: Date;
    private constructor();
    get id(): number;
    static createNew(internalCertificateId: number, payload: ITransferCommand): CertificateTransferredEvent;
}
export declare class CertificateClaimedEvent implements ICertificateEvent {
    private readonly _id;
    readonly internalCertificateId: number;
    readonly payload: IClaimCommand;
    readonly type = CertificateEventType.Claimed;
    readonly version = 1;
    readonly createdAt: Date;
    private constructor();
    get id(): number;
    static createNew(internalCertificateId: number, payload: IClaimCommand): CertificateClaimedEvent;
}
export declare class CertificateIssuancePersistedEvent implements ICertificateEvent {
    private readonly _id;
    readonly internalCertificateId: number;
    readonly payload: {
        blockchainCertificateId: number;
        persistedEventId: number;
        transactionHash: string;
    };
    readonly type = CertificateEventType.IssuancePersisted;
    readonly version = 1;
    readonly createdAt: Date;
    private constructor();
    get id(): number;
    static createNew(internalCertificateId: number, payload: CertificateIssuancePersistedEvent['payload']): CertificateIssuancePersistedEvent;
}
export declare class CertificateTransferPersistedEvent implements ICertificateEvent {
    private readonly _id;
    readonly internalCertificateId: number;
    readonly payload: {
        persistedEventId: number;
        transactionHash: string;
    };
    readonly type = CertificateEventType.TransferPersisted;
    readonly version = 1;
    readonly createdAt: Date;
    private constructor();
    get id(): number;
    static createNew(internalCertificateId: number, payload: CertificateTransferPersistedEvent['payload']): CertificateTransferPersistedEvent;
}
export declare class CertificateClaimPersistedEvent implements ICertificateEvent {
    private readonly _id;
    readonly internalCertificateId: number;
    readonly payload: {
        persistedEventId: number;
        transactionHash: string;
    };
    readonly type = CertificateEventType.ClaimPersisted;
    readonly version = 1;
    readonly createdAt: Date;
    private constructor();
    get id(): number;
    static createNew(internalCertificateId: number, payload: CertificateClaimPersistedEvent['payload']): CertificateClaimPersistedEvent;
}
export declare class CertificatePersistErrorEvent implements ICertificateEvent {
    private readonly _id;
    readonly internalCertificateId: number;
    readonly payload: {
        errorMessage: string;
        persistedEventId: number;
    };
    readonly type = CertificateEventType.PersistError;
    readonly version = 1;
    readonly createdAt: Date;
    private constructor();
    get id(): number;
    static createNew(internalCertificateId: number, payload: CertificatePersistErrorEvent['payload']): CertificatePersistErrorEvent;
}
