import { CertificateCommandRepository } from './repositories/CertificateCommand/CertificateCommand.repository';
import { CertificateEventRepository } from './repositories/CertificateEvent/CertificateEvent.repository';
import { EventBus } from '@nestjs/cqrs';
import { IClaimCommand, IIssueCommand, IIssueCommandParams, ITransferCommand } from '../types';
import { ICertificateReadModel, IClaimPersistedCommand, IIssuancePersistedCommand, IPersistErrorCommand, ITransferPersistedCommand } from './types';
import { CertificateReadModelRepository, IGetAllCertificatesOptions } from './repositories/CertificateReadModel/CertificateReadModel.repository';
import { CertificateEventService } from './repositories/CertificateEvent/CertificateEvent.service';
export declare class OffChainCertificateService<T = null> {
    private readonly certCommandRepo;
    private readonly certEventRepo;
    private readonly certificateEventService;
    private readonly eventBus;
    private readonly readModelRepo;
    constructor(certCommandRepo: CertificateCommandRepository, certEventRepo: CertificateEventRepository, certificateEventService: CertificateEventService, eventBus: EventBus, readModelRepo: CertificateReadModelRepository<T>);
    getAll(options?: IGetAllCertificatesOptions): Promise<ICertificateReadModel<T>[]>;
    getById(id: number): Promise<ICertificateReadModel<T> | null>;
    getByIds(ids: number[]): Promise<ICertificateReadModel<T>[]>;
    issue(params: IIssueCommandParams<T>): Promise<number>;
    issueWithoutValidation(command: IIssueCommand<T>): Promise<number>;
    claim(command: IClaimCommand): Promise<void>;
    claimWithoutValidation(command: IClaimCommand): Promise<void>;
    transfer(command: ITransferCommand): Promise<void>;
    transferWithoutValidation(command: ITransferCommand): Promise<void>;
    issuePersisted(internalCertificateId: number, command: IIssuancePersistedCommand): Promise<void>;
    claimPersisted(internalCertificateId: number, command: IClaimPersistedCommand): Promise<void>;
    transferPersisted(internalCertificateId: number, command: ITransferPersistedCommand): Promise<void>;
    persistError(internalCertificateId: number, command: IPersistErrorCommand): Promise<void>;
    batchIssue(originalCertificates: IIssueCommandParams<T>[]): Promise<number[]>;
    batchClaim(commands: IClaimCommand[]): Promise<void>;
    batchTransfer(commands: ITransferCommand[]): Promise<void>;
    private handleBatch;
    private validateBatchIssue;
    private validateBatchClaim;
    private validateBatchTransfer;
    private createAggregate;
    private generateInternalCertificateId;
    private propagate;
    private propagateMany;
}
