"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.OffChainCertificateService = void 0;
const common_1 = require("@nestjs/common");
const cqrs_1 = require("@nestjs/cqrs");
const Certificate_events_1 = require("./events/Certificate.events");
const certificate_aggregate_1 = require("./certificate.aggregate");
const errors_1 = require("./errors");
const repository_keys_1 = require("./repositories/repository.keys");
const CertificateEvent_service_1 = require("./repositories/CertificateEvent/CertificateEvent.service");
const validators_1 = require("./validators");
const batch_utils_1 = require("./utils/batch.utils");
let OffChainCertificateService = class OffChainCertificateService {
    constructor(certCommandRepo, certEventRepo, certificateEventService, eventBus, readModelRepo) {
        this.certCommandRepo = certCommandRepo;
        this.certEventRepo = certEventRepo;
        this.certificateEventService = certificateEventService;
        this.eventBus = eventBus;
        this.readModelRepo = readModelRepo;
    }
    async getAll(options = {}) {
        return await this.readModelRepo.getAll(options);
    }
    async getById(id) {
        return await this.readModelRepo.getByInternalCertificateId(id);
    }
    async getByIds(ids) {
        return await this.readModelRepo.getManyByInternalCertificateIds(ids);
    }
    async issue(params) {
        const command = {
            ...params,
            fromTime: Math.round(params.fromTime.getTime() / 1000),
            toTime: Math.round(params.toTime.getTime() / 1000)
        };
        await validators_1.validateIssueCommand(command);
        return await this.issueWithoutValidation(command);
    }
    async issueWithoutValidation(command) {
        const savedCommand = await this.certCommandRepo.save({ payload: command });
        const event = Certificate_events_1.CertificateIssuedEvent.createNew(await this.generateInternalCertificateId(), command);
        const aggregate = await this.createAggregate([event]);
        await this.propagate(event, savedCommand, aggregate);
        return event.internalCertificateId;
    }
    async claim(command) {
        await validators_1.validateClaimCommand(command);
        return await this.claimWithoutValidation(command);
    }
    async claimWithoutValidation(command) {
        const savedCommand = await this.certCommandRepo.save({ payload: command });
        const event = Certificate_events_1.CertificateClaimedEvent.createNew(command.certificateId, command);
        const aggregate = await this.createAggregate([event]);
        await this.propagate(event, savedCommand, aggregate);
    }
    async transfer(command) {
        await validators_1.validateTransferCommand(command);
        return await this.transferWithoutValidation(command);
    }
    async transferWithoutValidation(command) {
        const savedCommand = await this.certCommandRepo.save({ payload: command });
        const event = Certificate_events_1.CertificateTransferredEvent.createNew(command.certificateId, command);
        const aggregate = await this.createAggregate([event]);
        await this.propagate(event, savedCommand, aggregate);
    }
    async issuePersisted(internalCertificateId, command) {
        const savedCommand = await this.certCommandRepo.save({ payload: command });
        const event = Certificate_events_1.CertificateIssuancePersistedEvent.createNew(internalCertificateId, command);
        const aggregate = await this.createAggregate([event]);
        await this.propagate(event, savedCommand, aggregate);
    }
    async claimPersisted(internalCertificateId, command) {
        const savedCommand = await this.certCommandRepo.save({ payload: command });
        const event = Certificate_events_1.CertificateClaimPersistedEvent.createNew(internalCertificateId, command);
        const aggregate = await this.createAggregate([event]);
        await this.propagate(event, savedCommand, aggregate);
    }
    async transferPersisted(internalCertificateId, command) {
        const savedCommand = await this.certCommandRepo.save({ payload: command });
        const event = Certificate_events_1.CertificateTransferPersistedEvent.createNew(internalCertificateId, command);
        const aggregate = await this.createAggregate([event]);
        await this.propagate(event, savedCommand, aggregate);
    }
    async persistError(internalCertificateId, command) {
        const savedCommand = await this.certCommandRepo.save({ payload: command });
        const event = Certificate_events_1.CertificatePersistErrorEvent.createNew(internalCertificateId, command);
        const aggregate = await this.createAggregate([event]);
        await this.propagate(event, savedCommand, aggregate, command.errorMessage);
    }
    async batchIssue(originalCertificates) {
        if (!originalCertificates.length) {
            return [];
        }
        const commands = originalCertificates.map((c) => ({
            ...c,
            fromTime: Math.round(c.fromTime.getTime() / 1000),
            toTime: Math.round(c.toTime.getTime() / 1000)
        }));
        await validators_1.validateBatchIssueCommands(commands);
        await this.validateBatchIssue(commands);
        const savedCommands = await this.certCommandRepo.saveMany(commands.map((command) => ({ payload: command })));
        const events = await batch_utils_1.createIssueEventsFromCommands(commands, () => this.generateInternalCertificateId());
        const eventsByCertificateId = batch_utils_1.groupByInternalCertificateId(events);
        const aggregates = await batch_utils_1.createAggregatesFromCertificateGroups(eventsByCertificateId, (events) => this.createAggregate(events));
        await this.propagateMany(batch_utils_1.zipEventsWithCommandId(events, savedCommands), aggregates);
        return events.map((event) => event.internalCertificateId);
    }
    async batchClaim(commands) {
        if (!commands.length) {
            return;
        }
        await validators_1.validateBatchClaimCommands(commands);
        await this.validateBatchClaim(commands);
        await this.handleBatch(commands);
    }
    async batchTransfer(commands) {
        if (!commands.length) {
            return;
        }
        await validators_1.validateBatchTransferCommands(commands);
        await this.validateBatchTransfer(commands);
        await this.handleBatch(commands);
    }
    async handleBatch(commands) {
        const savedCommands = await this.certCommandRepo.saveMany(commands.map((command) => ({ payload: command })));
        const events = commands.map((command) => batch_utils_1.createEventFromCommand(command));
        const eventsByCertificate = batch_utils_1.groupByInternalCertificateId(events);
        const aggregates = await batch_utils_1.createAggregatesFromCertificateGroups(eventsByCertificate, (events) => this.createAggregate(events));
        await this.propagateMany(batch_utils_1.zipEventsWithCommandId(events, savedCommands), aggregates);
    }
    async validateBatchIssue(commands) {
        try {
            commands.forEach((c) => {
                certificate_aggregate_1.CertificateAggregate.fromEvents([Certificate_events_1.CertificateIssuedEvent.createNew(-1, c)]);
            });
        }
        catch (err) {
            throw new errors_1.CertificateErrors.BatchError(err);
        }
    }
    async validateBatchClaim(commands) {
        const events = commands.map((command) => batch_utils_1.createEventFromCommand(command));
        const eventsByCertificateId = batch_utils_1.groupByInternalCertificateId(events);
        try {
            await batch_utils_1.createAggregatesFromCertificateGroups(eventsByCertificateId, (events) => this.createAggregate(events));
        }
        catch (err) {
            throw new errors_1.CertificateErrors.BatchError(err);
        }
    }
    async validateBatchTransfer(commands) {
        const events = commands.map((command) => batch_utils_1.createEventFromCommand(command));
        const eventsByCertificateId = batch_utils_1.groupByInternalCertificateId(events);
        try {
            await batch_utils_1.createAggregatesFromCertificateGroups(eventsByCertificateId, (events) => this.createAggregate(events));
        }
        catch (err) {
            throw new errors_1.CertificateErrors.BatchError(err);
        }
    }
    async createAggregate(newEvents) {
        const certificateId = newEvents[0].internalCertificateId;
        const previousEvents = await this.certEventRepo.getByInternalCertificateId(certificateId);
        return certificate_aggregate_1.CertificateAggregate.fromEvents([...previousEvents, ...newEvents]);
    }
    async generateInternalCertificateId() {
        return await this.certEventRepo.getNextInternalCertificateId();
    }
    async propagate(event, command, aggregate, errorMessage) {
        await this.eventBus.publish(event);
        await this.readModelRepo.save(aggregate.getCertificate());
        const savedEvent = await this.certificateEventService.save(event, command.id);
        if (Certificate_events_1.isPersistedEvent(savedEvent)) {
            await this.certEventRepo.updateAttempt({
                eventIds: [savedEvent.payload.persistedEventId],
                error: errorMessage
            });
        }
    }
    async propagateMany(events, aggregates, errorMessage) {
        await this.eventBus.publishAll(events);
        await this.readModelRepo.saveMany(aggregates.map((aggregate) => aggregate.getCertificate()));
        const savedEvents = await this.certificateEventService.saveMany(events);
        const persistedEvents = savedEvents.filter((event) => Certificate_events_1.isPersistedEvent(event));
        await this.certEventRepo.updateAttempt({
            eventIds: persistedEvents.map((event) => event.payload.persistedEventId),
            error: errorMessage
        });
    }
};
OffChainCertificateService = __decorate([
    common_1.Injectable(),
    __param(0, common_1.Inject(repository_keys_1.CERTIFICATE_COMMAND_REPOSITORY)),
    __param(1, common_1.Inject(repository_keys_1.CERTIFICATE_EVENT_REPOSITORY)),
    __param(4, common_1.Inject(repository_keys_1.CERTIFICATE_READ_MODEL_REPOSITORY)),
    __metadata("design:paramtypes", [Object, Object, CertificateEvent_service_1.CertificateEventService,
        cqrs_1.EventBus, Object])
], OffChainCertificateService);
exports.OffChainCertificateService = OffChainCertificateService;
//# sourceMappingURL=offchain-certificate.service.js.map