"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CertificateEventInMemoryRepository = void 0;
const common_1 = require("@nestjs/common");
const Certificate_events_1 = require("../../events/Certificate.events");
const array_1 = require("../../utils/array");
const lodash_1 = require("lodash");
let CertificateEventInMemoryRepository = class CertificateEventInMemoryRepository {
    constructor() {
        this.internalCertificateIdSerial = 0;
        this.eventDb = [];
        this.attemptDb = {};
        this.maxSynchronizationAttemptsForEvent = Infinity;
    }
    async getAll() {
        return this.eventDb;
    }
    async getByInternalCertificateId(internalCertId) {
        return this.eventDb.filter((e) => e.internalCertificateId === internalCertId);
    }
    async save(event, commandId) {
        const entity = {
            internalCertificateId: event.internalCertificateId,
            type: event.type,
            version: event.version,
            payload: event.payload,
            createdAt: event.createdAt,
            commandId: commandId,
            id: this.eventDb.length + 1
        };
        this.eventDb.push(entity);
        return entity;
    }
    async saveMany(events) {
        const saved = [];
        events.forEach((event) => {
            const entity = {
                internalCertificateId: event.internalCertificateId,
                type: event.type,
                version: event.version,
                payload: event.payload,
                createdAt: event.createdAt,
                commandId: event.commandId,
                id: this.eventDb.length + 1
            };
            this.eventDb.push(entity);
            saved.push(entity);
        });
        return saved;
    }
    async createSynchronizationAttempt(eventId) {
        const entity = {
            eventId,
            attemptsCount: 0,
            createdAt: new Date()
        };
        this.attemptDb[eventId] = entity;
        return entity;
    }
    async createSynchronizationAttempts(eventIds) {
        const savedEntities = [];
        eventIds.forEach((eventId) => {
            const entity = {
                eventId,
                attemptsCount: 0,
                createdAt: new Date()
            };
            this.attemptDb[eventId] = entity;
            savedEntities.push(entity);
        });
        return savedEntities;
    }
    async updateAttempt({ eventIds, error }) {
        const savedAttempts = [];
        eventIds.forEach((eventId) => {
            const synchronizationAttempt = this.attemptDb[eventId];
            if (!synchronizationAttempt) {
                throw new Error('Synchronization attempt does not exist');
            }
            synchronizationAttempt.attemptsCount = synchronizationAttempt.attemptsCount + 1;
            synchronizationAttempt.error = error ?? null;
            savedAttempts.push(synchronizationAttempt);
        });
        return savedAttempts;
    }
    async findAllToProcess() {
        const failedEventIds = Object.values(this.attemptDb)
            .filter((a) => a.error && a.attemptsCount > this.maxSynchronizationAttemptsForEvent)
            .map((a) => a.eventId);
        const failedCerts = lodash_1.chain(this.eventDb)
            .filter((e) => failedEventIds.includes(e.id))
            .map((e) => e.internalCertificateId)
            .compact()
            .uniq()
            .value();
        const attempts = Object.values(this.attemptDb).filter((a) => {
            return ((!a.error && a.attemptsCount === 0) ||
                (a.error && a.attemptsCount < this.maxSynchronizationAttemptsForEvent));
        });
        const events = lodash_1.chain(attempts)
            .map((a) => this.eventDb.find((e) => e.id === a.eventId))
            .compact()
            .filter((e) => !failedCerts.includes(e.internalCertificateId))
            .value();
        return events;
    }
    async getOne(eventId) {
        const event = this.eventDb.find((e) => e.id === eventId);
        if (!event) {
            throw new common_1.NotFoundException();
        }
        return event;
    }
    async getBlockchainIdMap(internalCertIds) {
        const issuanceEvents = this.eventDb.filter(Certificate_events_1.isIssuePersistedEvent);
        const events = issuanceEvents.filter((e) => internalCertIds.includes(e.internalCertificateId));
        const idMap = array_1.arrayToMap(events, (e) => e.internalCertificateId, (e) => e.payload.blockchainCertificateId);
        return idMap;
    }
    async getNextInternalCertificateId() {
        this.internalCertificateIdSerial += 1;
        return this.internalCertificateIdSerial;
    }
};
CertificateEventInMemoryRepository = __decorate([
    common_1.Injectable()
], CertificateEventInMemoryRepository);
exports.CertificateEventInMemoryRepository = CertificateEventInMemoryRepository;
//# sourceMappingURL=CertificateEventInMemory.repository.js.map