"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BlockchainPropertiesService = void 0;
const common_1 = require("@nestjs/common");
const issuer_1 = require("@energyweb/issuer");
const config_1 = require("@nestjs/config");
const deploymentProperties_repository_1 = require("./repositories/deploymentProperties/deploymentProperties.repository");
const ethers_1 = require("ethers");
const utils_general_1 = require("@energyweb/utils-general");
const wait_utils_1 = require("../utils/wait.utils");
let BlockchainPropertiesService = class BlockchainPropertiesService {
    constructor(configService, deploymentPropsRepo) {
        this.configService = configService;
        this.deploymentPropsRepo = deploymentPropsRepo;
        this.assure0x = (privateKey) => privateKey.startsWith('0x') ? privateKey : `0x${privateKey}`;
        const [primaryRPC, fallbackRPC] = this.configService.get('WEB3').split(';');
        const issuerPrivateKey = this.configService.get('ISSUER_PRIVATE_KEY');
        if (!primaryRPC) {
            throw new Error('No WEB3 environment variable set');
        }
        if (!issuerPrivateKey) {
            throw new Error('No ISSUER_PRIVATE_KEY environment variable set');
        }
        this.primaryRPC = primaryRPC;
        this.issuerPrivateKey = issuerPrivateKey;
        this.fallbackRPC = fallbackRPC ? fallbackRPC : null;
    }
    async get() {
        await wait_utils_1.waitForState(async () => await this.isDeployed(), 'Blockchain properties were not deployed', { interval: 5000, maxTries: 24 });
        const { registry, issuer } = await this.deploymentPropsRepo.get();
        return {
            rpcNode: this.primaryRPC,
            fallbackRpcNode: this.fallbackRPC,
            issuer,
            registry
        };
    }
    async getWrapped() {
        await wait_utils_1.waitForState(async () => await this.isDeployed(), 'Blockchain properties were not deployed', { interval: 5000, maxTries: 24 });
        const { registry, issuer } = await this.deploymentPropsRepo.get();
        const web3 = utils_general_1.getProviderWithFallback(...[this.primaryRPC, this.fallbackRPC ?? ''].filter((url) => Boolean(url)));
        const signer = new ethers_1.Wallet(this.assure0x(this.issuerPrivateKey), web3);
        return {
            web3,
            registry: issuer_1.Contracts.factories.RegistryExtendedFactory.connect(registry, signer),
            issuer: issuer_1.Contracts.factories.IssuerFactory.connect(issuer, signer),
            activeUser: signer
        };
    }
    async deploy() {
        if (await this.deploymentPropsRepo.propertiesExist()) {
            return;
        }
        const provider = new ethers_1.providers.FallbackProvider([
            new ethers_1.providers.JsonRpcProvider(this.primaryRPC)
        ]);
        const registry = await issuer_1.Contracts.migrateRegistry(provider, this.issuerPrivateKey);
        const issuer = await issuer_1.Contracts.migrateIssuer(provider, this.issuerPrivateKey, registry.address);
        await this.deploymentPropsRepo.save({ registry: registry.address, issuer: issuer.address });
    }
    async isDeployed() {
        return await this.deploymentPropsRepo.propertiesExist();
    }
    async wrap(privateKey) {
        const { registry, issuer } = await this.deploymentPropsRepo.get();
        const { web3 } = await this.getWrapped();
        const signer = new ethers_1.Wallet(this.assure0x(privateKey), web3);
        return {
            web3,
            registry: issuer_1.Contracts.factories.RegistryExtendedFactory.connect(registry, signer),
            issuer: issuer_1.Contracts.factories.IssuerFactory.connect(issuer, signer),
            activeUser: signer
        };
    }
};
BlockchainPropertiesService = __decorate([
    common_1.Injectable(),
    __metadata("design:paramtypes", [config_1.ConfigService,
        deploymentProperties_repository_1.DeploymentPropertiesRepository])
], BlockchainPropertiesService);
exports.BlockchainPropertiesService = BlockchainPropertiesService;
//# sourceMappingURL=blockchain-properties.service.js.map