"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var CertificateOperationsService_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.CertificateOperationsService = void 0;
const common_1 = require("@nestjs/common");
const types_1 = require("../types");
const transaction_poll_service_1 = require("./transaction-poll.service");
const batch_claim_certificates_handler_1 = require("./claim/batch-claim-certificates.handler");
const batch_issue_certificates_handler_1 = require("./issue/batch-issue-certificates.handler");
const batch_transfer_certificates_handler_1 = require("./transfer/batch-transfer-certificates.handler");
const claim_certificate_handler_1 = require("./claim/claim-certificate.handler");
const issue_certificate_handler_1 = require("./issue/issue-certificate.handler");
const transfer_certificate_handler_1 = require("./transfer/transfer-certificate.handler");
let CertificateOperationsService = CertificateOperationsService_1 = class CertificateOperationsService {
    constructor(transactionPoll, batchClaimCertificatesHandler, batchIssueCertificatesHandler, batchTransferCertificatesHandler, claimCertificateHandler, issueCertificateHandler, transferCertificateHandler) {
        this.transactionPoll = transactionPoll;
        this.batchClaimCertificatesHandler = batchClaimCertificatesHandler;
        this.batchIssueCertificatesHandler = batchIssueCertificatesHandler;
        this.batchTransferCertificatesHandler = batchTransferCertificatesHandler;
        this.claimCertificateHandler = claimCertificateHandler;
        this.issueCertificateHandler = issueCertificateHandler;
        this.transferCertificateHandler = transferCertificateHandler;
        this.logger = new common_1.Logger(CertificateOperationsService_1.name);
    }
    async process({ data }) {
        switch (data.type) {
            case types_1.BlockchainActionType.Issuance:
                return await this.issue(data.payload);
            case types_1.BlockchainActionType.Transfer:
                return await this.transfer(data.payload);
            case types_1.BlockchainActionType.Claim:
                return await this.claim(data.payload);
            case types_1.BlockchainActionType.BatchIssuance:
                return await this.batchIssue(data.payload);
            case types_1.BlockchainActionType.BatchTransfer:
                return await this.batchTransfer(data.payload);
            case types_1.BlockchainActionType.BatchClaim:
                return await this.batchClaim(data.payload);
        }
    }
    async issue(params) {
        this.logger.debug(`Triggering issuance for: ${JSON.stringify(params)}`);
        const issuanceTx = await this.issueCertificateHandler.execute(params);
        const [certificateId] = await this.transactionPoll.waitForNewCertificates(issuanceTx.hash);
        return {
            certificateId,
            transactionHash: issuanceTx.hash
        };
    }
    async transfer(params) {
        this.logger.debug(`Triggering transfer for: ${JSON.stringify(params)}`);
        const transferTx = await this.transferCertificateHandler.execute(params);
        await this.transactionPoll.waitForTransaction(transferTx.hash);
        return { transactionHash: transferTx.hash };
    }
    async claim(params) {
        this.logger.debug(`Triggering claim for: ${JSON.stringify(params)}`);
        const claimTx = await this.claimCertificateHandler.execute(params);
        await this.transactionPoll.waitForTransaction(claimTx.hash);
        return { transactionHash: claimTx.hash };
    }
    async batchIssue(params) {
        this.logger.debug(`Triggering batch issuance for: ${JSON.stringify(params)}`);
        const batchIssuanceTx = await this.batchIssueCertificatesHandler.execute({
            certificates: params.certificates
        });
        const batchIssuanceCertificateIds = await this.transactionPoll.waitForNewCertificates(batchIssuanceTx.hash);
        return {
            certificateIds: batchIssuanceCertificateIds,
            transactionHash: batchIssuanceTx.hash
        };
    }
    async batchTransfer(params) {
        this.logger.debug(`Triggering batch transfer for: ${JSON.stringify(params)}`);
        const batchTransferTx = await this.batchTransferCertificatesHandler.execute({
            transfers: params.transfers
        });
        await this.transactionPoll.waitForTransaction(batchTransferTx.hash);
        return {
            transactionHash: batchTransferTx.hash
        };
    }
    async batchClaim(params) {
        this.logger.debug(`Triggering batch claim for: ${JSON.stringify(params)}`);
        const batchClaimTx = await this.batchClaimCertificatesHandler.execute({
            claims: params.claims
        });
        await this.transactionPoll.waitForTransaction(batchClaimTx.hash);
        return {
            transactionHash: batchClaimTx.hash
        };
    }
};
CertificateOperationsService = CertificateOperationsService_1 = __decorate([
    common_1.Injectable(),
    __metadata("design:paramtypes", [transaction_poll_service_1.TransactionPollService,
        batch_claim_certificates_handler_1.BatchClaimCertificatesHandler,
        batch_issue_certificates_handler_1.BatchIssueCertificatesHandler,
        batch_transfer_certificates_handler_1.BatchTransferCertificatesHandler,
        claim_certificate_handler_1.ClaimCertificateHandler,
        issue_certificate_handler_1.IssueCertificateHandler,
        transfer_certificate_handler_1.TransferCertificateHandler])
], CertificateOperationsService);
exports.CertificateOperationsService = CertificateOperationsService;
//# sourceMappingURL=certificate-operations.service.js.map